@@ -66,6 +66,15 @@ constexpr const Color::Modifier yellow(Color::FG_YELLOW);
 OPENDHT_PUBLIC void
 printLog(std::ostream &s, char const *m, va_list args);
 
+OPENDHT_PUBLIC
+std::unique_ptr<Logger> getStdLogger();
+
+OPENDHT_PUBLIC
+std::unique_ptr<Logger> getFileLogger(const std::string &path);
+
+OPENDHT_PUBLIC
+std::unique_ptr<Logger> getSyslogLogger(const char* name);
+
 OPENDHT_PUBLIC void
 enableLogging(dht::DhtRunner &dht);
 
